
use warnings;
use strict;

package Mozilla::Persona::Validate::Table;
use base 'Mozilla::Persona::Validate';

use Log::Report    qw/persona/;
use Digest         ();

=chapter NAME
Mozilla::Persona::Validate::Table - validate the existence via a simple table

=chapter SYNOPSIS
  # add to the configation of $domain.json
  { ...
    "validate" : {
      "class"  : "Mozilla::Persona::Validator::Table",
      "pwfile" : "/etc/persona/passwd",
    },
    ...
  }

=chapter DESCRIPTION
The validator object must ensure that the user/password combination
is valid.  In this case, a simple password-file-like table is used
to authenticate the user.

You may put that file with the other configuration information
in F</etc/persona/>, for instance under the name C<passwd>.

=section Password file syntax

Lines which are blank (visually empty) or start with a '#' (comment)
are ignored.  Other lines describe users.

The user definition lines contain (currently) three fields separated
by a colon ':'.  First a username, than the digest used (acceptable to
M<Digest>) and finally the base64 encoded version of that digest.

For instance:

  # my test user
  test:md5:47b0eb54a2f09e1b97881eb8746486ce

=chapter METHODS

=section Constructors

=c_method new OPTIONS
=requires pwfile FILENAME
The password file.  Probably, you want to provide an absolute path.
When the file changes, it will automatically be reloaded.
=cut

sub init($)
{   my ($self, $args) = @_;

    my $fn = $args->{pwfile} or panic;
    $self->openFile($fn);   # pre-load
    $self;
}

#------------
=section Accessors
=method pwfile
=cut

sub pwfile() {shift->{MPVT_fn}}

sub openFile(;$)
{   my ($self, $fn) = @_;

    if($fn) { $self->{MPVT_fn} = $fn }
    else    { $fn = $self->{MPVT_fn} }

    my $mtime = (stat $fn)[9];
    defined $mtime
        or fault __x"passwd file {fn}";

    if(my $last_mtime = $self->{MPVT_mtime})
    {   return $self->{MPVT_info} if $mtime eq $last_mtime;
    }

    my %info;
    open PASSWD, '<:raw', $fn
        or fault __x"cannot read file {file}", file => $fn;

    while(<PASSWD>)
    {   next if m/^#|^\s*$/;
        my ($user, $algo, $passwd) = split /\:/;

        defined $passwd
            or warning __x"not enough fields in {file} line {linenr}"
                 , file => $fn, linenr => $.;

        !$info{$user}
            or warning __x"found user {user} again in {file} line {linenr}"
                 , file => $fn, linenr => $.;

        $info{$user} = [$algo, $passwd];
    }
    close PASSWD
        or fault __x"read errors in {file}", file => $fn;

    $self->{MPVT_mtime} = $mtime;
    $self->{MPVT_info}  = \%info;
}

sub isValid($$)
{   my ($self, $user, $password) = @_;

    my $info = $self->openFile->{$user}
        or return 0;   # unknown user
   
    my ($algo, $expect) = @$info;
    my $digester = eval { Digest->new($algo) };
    error __x"unsupported digest algorithm {name} for {user}"
      , name => $algo, user => $user;

    $password eq $digester->add($password)->b64digest;
}

1;
